%global __spec_install_pre %{___build_pre}

Summary: The Linux Realtime kernel

# mrgN
%global iteration 34

# major and minor kernel version
# e.g. 3.2.4 has a major_kver of 3 and a minor_kver of 2
%global major_kver	3
%global minor_kver	10

# Do we have a -stable update to apply?
%global stable_update 33

# The preempt RT patch level
%global rttag %{!?rt_patch_version:rt32}

# conditional with/without variables
# Note that the logic here is inverted; a  bcond_without implies
# that the variable is on by default (since you use --without to turn it off)
# Likewise a bcond_with implies the variable is off by default (turned on by --with)
%bcond_without rt
%bcond_without doc
%bcond_without debug
%bcond_with    headers
%bcond_without vanilla
%bcond_without trace
%bcond_with    perf
%bcond_without firmware
%bcond_with    virt
%bcond_without debuginfo
%bcond_without dracut
%bcond_with    release

# What parts do we want to build?  We must build at least one kernel.
# These are the kernels that are built IF the architecture allows it.
# Note that these are regular macros, not the bcond macros defined above,
# so they can be reassigned inside the spec file.

%global buildrt        %{with rt}
%global builddoc       %{with doc}
%global builddebug     %{with debug}
%global buildheaders   %{with headers}
%global buildvanilla   %{with vanilla}
%global buildtrace     %{with trace}
%global buildperf      %{with perf}
%global buildfirmware  %{with firmware}
%global buildvirt      %{with virt}
%global builddebuginfo %{with debuginfo}
%global buildrelease   %{with release}

#
# this is the MRG release number which will go into
#       /etc/mrg-realtime-release
# The format is <major>.<minor>.<errata>

%if %{buildrelease}
%global mrg_major	2
%global mrg_minor	5
%global mrg_errata	0
%global mrg_release  	%{mrg_major}.%{mrg_minor}.%{mrg_errata}
%endif

# use dracut or mkinitrd?
%global use_dracut    %{with dracut}

# For a stable, released kernel, released_kernel should be 1. For rawhide
# and/or a kernel built from an rc or git snapshot, released_kernel should
# be 0.
%global released_kernel 1

# Versions of various parts

# Polite request for people who spin their own kernel rpms:
# please modify the "buildid" define in a way that identifies
# that the kernel isn't the stock distribution kernel, for example,
# by setting the define to ".local" or ".bz123456"
#
# % define buildid .local

%global rhel 1
%if %{rhel}
%global distro_build  0
### %global signmodules 1
%global signmodules 0
%else
# fedora_build defines which build revision of this kernel version we're
# building. Rather than incrementing forever, as with the prior versioning
# setup, we set fedora_cvs_origin to the current cvs revision s/1.// of the
# kernel spec when the kernel is rebased, so fedora_build automatically
# works out to the offset from the rebase, so it doesn't get too ginormous.
#
# If you're building on a branch, the RCS revision will be something like
# 1.1205.1.1.  In this case we drop the initial 1, subtract fedora_cvs_origin
# from the second number, and then append the rest of the RCS string as is.
# Don't stare at the awk too long, you'll go blind.
%global fedora_cvs_origin   1462
%global fedora_cvs_revision() %2
%global distro_build %(echo %{fedora_cvs_origin}.%{fedora_cvs_revision $Revision: 1.694 $} | awk -F . '{ OFS = "."; ORS = ""; print $3 - $1 ; i = 4 ; OFS = ""; while (i <= NF) { print ".", $i ; i++} }')
%global distro_build %{fedora_build}
%global signmodules 0
%endif

# if patch fuzzy patch applying will be forbidden
%global with_fuzzy_patches 0

## If this is a released kernel ##
%if 0%{?released_kernel}

# Fix a misnaming in later RT patches
%global stable_minor_rev 0
%if 0%{?stable_minor_rev}
%global stable_minor .%{stable_minor_rev}
%endif

# Is it a -stable RC?
%global stable_rc 0
# Set rpm version accordingly
%if 0%{?stable_update}
%global stablerev .%{stable_update}
%global stable_base %{stable_update}
%if 0%{?stable_rc}
# stable RCs are incremental patches, so we need the previous stable patch
%global stable_base %(echo $((%{stable_update} - 1)))
%endif
%endif
%global rpmversion %{major_kver}.%{minor_kver}%{?stablerev}%{?stable_minor}

## The not-released-kernel case ##
%else
# The next upstream release sublevel (minor_kver+1)
%global upstream_sublevel %(echo $((%{minor_kver} + 1)))
# The rc snapshot level
%global rcrev 0
# The git snapshot level
%global gitrev 0
# Set rpm version accordingly
%global rpmversion %{major_kver}.%{upstream_sublevel}
%endif
# Nb: The above rcrev and gitrev values automagically define Patch00 and Patch01 below.

# What parts do we want to build?  We must build at least one kernel.
# These are the kernels that are built IF the architecture allows it.
# All should default to 1 (enabled) and be flipped to 0 (disabled)
# by later arch-specific checks.

# The following build options are enabled by default.
# Use either --without <opt> in your rpmbuild command or force values
# to 0 in here to disable them.
#
# perf noarch subpkg
%global with_vdso_install 1

# Build the kernel-doc package, but don't fail the build if it botches.
# Here "true" means "continue" and "false" means "fail the build".
%if 0%{?released_kernel}
%global doc_build_fail false
%else
%global doc_build_fail true
%endif

# Additional options for user-friendly one-off kernel building:
# Only build the base kernel (--with baseonly):
%global with_baseonly  0
# Only build the debug kernel (--with dbgonly):
%global with_dbgonly   0

# should we do C=1 builds with sparse
%global with_sparse	%{?_with_sparse:      1} %{?!_with_sparse:      0}

# pkg_release is what we'll fill in for the rpm Release: field
%if 0%{?released_kernel}

%if 0%{?stable_rc}
%global stable_rctag .rc%{stable_rc}
%endif

%global pkg_release_simple %{rttag}.%{iteration}%{?stable_rctag}

%else

# non-released_kernel
%if 0%{?rcrev}
%global rctag .rc%rcrev
%else
%global rctag .rc0
%endif
%if 0%{?gitrev}
%global gittag .git%gitrev
%else
%global gittag .git0
%endif

%global pkg_release_simple %{rttag}.%{iteration}.0.%{?rctag}%{?gittag}
%endif

# The kernel tarball/base version
%global kversion %{major_kver}.%{minor_kver}.%{stable_update}

%global make_target bzImage

%global hdrarch %_target_cpu
%global asmarch %_target_cpu

%global using_upstream_branch 0
%if 0%{?upstream_branch:1}
%global stable_update 0
%global using_upstream_branch 1
%global variant -%{upstream_branch}%{?variant_fedora}
%global pkg_release_simple %{rttag}.%{iteration}.0.%{upstream_branch_tag}
%endif

%if %{rhel}
%global pkg_release_simple %{rttag}.%{iteration}
%endif

%global pkg_release %{pkg_release_simple}%{?buildid}.el6rt
%global pkg_release_simple %{pkg_release_simple}.el6rt
%global KVERREL %{rpmversion}-%{pkg_release}.%{_target_cpu}

%if !%{builddebuginfo}
%global _enable_debug_packages 0
%global fancy_debuginfo 0
%else
%global _enable_debug_packages 1
%global fancy_debuginfo 1
%endif
%global debuginfodir /usr/lib/debug

# if requested, only build base kernel
%if %{with_baseonly}
%global builddebug 0
%global buildtrace 0
%global buildvanilla 0
%global buildvirt 0
%global buildperf 0
%global buildheaders 0
%global builddebuginfo 0
%endif

# if requested, only build debug kernel
%if %{with_dbgonly}
%if %{debugbuildsenabled}
%global buildrt 0
%endif
%global buildperf 0
%endif

%global all_x86 i386 i686

%if %{with_vdso_install}
# These arches install vdso/ directories.
%global vdso_arches %{all_x86} x86_64
%endif


# don't do debug builds on anything but i686 and x86_64
%ifnarch i686 x86_64
%global builddebug 0
%endif

# only package docs if we're noarch
%ifnarch noarch
%global builddoc 0
%global buildfirmware 0
%global buildrelease 0
%endif

# don't build noarch kernels or headers (duh)
%ifarch noarch
%global buildrt 0
%global buildheaders 0
%global buildperf 0
%global buildvirt 0
%global buildtrace 0
%global buildvanilla 0
%global all_arch_configs kernel-rt-%{version}-*.config
%endif

# Per-arch tweaks

%global asmarch x86
%global image_install_path boot
%global kernel_image arch/x86/boot/bzImage

%ifarch %{all_x86}
%global hdrarch i386
### %global all_arch_configs kernel-%{version}-i?86*.config
%global all_arch_configs kernel-rt-%{rpmversion}-i?86-rt*.config
%endif

%ifarch x86_64
### %global all_arch_configs kernel-%{version}-x86_64*.config
%global all_arch_configs kernel-rt-%{rpmversion}-x86_64-rt*.config
%endif

#%global oldconfig_target nonint_oldconfig
%global oldconfig_target oldconfig

# To temporarily exclude an architecture from being built, add it to
# %nobuildarches. Do _NOT_ use the ExclusiveArch: line, because if we
# don't build kernel-headers then the new build system will no longer let
# us use the previous build of that package -- it'll just be completely AWOL.
# Which is a BadThing(tm).

# We don't build a kernel on i386; we only do kernel-headers there,
# and we no longer build for 31bit S390. Same for 32bit sparc and arm.
%global nobuildarches i386 i586 i686 s390 sparc sparc64 ppc ia64 %{arm} noarch

%ifarch %nobuildarches
%global buildrt 0
%global builddebug 0
%global buildvanilla 0
%global buildtrace 0
%global buildvirt 0
%global builddebuginfo 0
%global buildperf 0
%global _enable_debug_packages 0
%endif

#
# Three sets of minimum package version requirements in the form of Conflicts:
# to versions below the minimum
#

#
# First the general kernel 3.0 required versions as per
# Documentation/Changes
#
%global kernel_dot_org_conflicts  ppp < 2.4.3-3, isdn4k-utils < 3.2-32, nfs-utils < 1.0.7-12, e2fsprogs < 1.37-4, util-linux < 2.12, jfsutils < 1.1.7-2, reiserfs-utils < 3.6.19-2, xfsprogs < 2.6.13-4, procps < 3.2.5-6.3, oprofile < 0.9.1-2

#
# Then a series of requirements that are distribution specific, either
# because we add patches for something, or the older versions have
# problems with the newer kernel or lack certain things that make
# integration in the distro harder than needed.
#
%global package_conflicts initscripts < 7.23, udev < 145-11, iptables < 1.3.2-1, ipw2200-firmware < 2.4, iwl4965-firmware < 228.57.2, selinux-policy-targeted < 1.25.3-14, squashfs-tools < 4.0, wireless-tools < 29-3

# We moved the drm include files into kernel-headers, make sure there's
# a recent enough libdrm-devel on the system that doesn't have those.
%global kernel_headers_conflicts libdrm-devel < 2.4.0-0.15

#
# Packages that need to be installed before the kernel is, because the %post
# scripts use them.
#
%global kernel_prereq  fileutils, module-init-tools, initscripts >= 8.11.1-1, kernel-rt-firmware >= %{rpmversion}-%{pkg_release}, mrg-rt-release >= %{rpmversion}-%{pkg_release}, grubby >= 7.0.4-1, rt-setup >= 1.55-1, rt-firmware >= 1.4-4

%if %{use_dracut}
%global initrd_prereq  dracut-kernel >= 002-18.git413bcf78
%else
%global initrd_prereq  mkinitrd >= 6.0.61-1
%endif

#
# This macro does requires, provides, conflicts, obsoletes for a kernel package.
#	%%kernel_reqprovconf <subpackage>
# It uses any kernel_<subpackage>_conflicts and kernel_<subpackage>_obsoletes
# macros defined above.
#
%define kernel_reqprovconf \
Provides: kernel = %{rpmversion}-%{pkg_release}\
Provides: kernel-%{_target_cpu} = %{rpmversion}-%{pkg_release}%{?1:.%{1}}\
Provides: kernel-drm = 4.3.0\
Provides: kernel-drm-nouveau = 16\
Provides: kernel-modeset = 1\
Provides: kernel-uname-r = %{KVERREL}%{?1:.%{1}}\
Requires(pre): %{kernel_prereq}\
Requires(pre): %{initrd_prereq}\
Requires(post): /sbin/new-kernel-pkg\
Requires(preun): /sbin/new-kernel-pkg\
Conflicts: %{kernel_dot_org_conflicts}\
Conflicts: %{package_conflicts}\
%{expand:%%{?kernel%{?1:_%{1}}_conflicts:Conflicts: %%{kernel%{?1:_%{1}}_conflicts}}}\
%{expand:%%{?kernel%{?1:_%{1}}_obsoletes:Obsoletes: %%{kernel%{?1:_%{1}}_obsoletes}}}\
%{expand:%%{?kernel%{?1:_%{1}}_provides:Provides: %%{kernel%{?1:_%{1}}_provides}}}\
# We can't let RPM do the dependencies automatic because it'll then pick up\
# a correct but undesirable perl dependency from the module headers which\
# isn't required for the kernel proper to function\
AutoReq: no\
AutoProv: yes\
%{nil}

Name: kernel-rt
Group: System Environment/Kernel
License: GPLv2
URL: http://www.kernel.org/
Version: %{rpmversion}
Release: %{pkg_release}
# DO NOT CHANGE THE 'ExclusiveArch' LINE TO TEMPORARILY EXCLUDE AN ARCHITECTURE BUILD.
# SET %%nobuildarches (ABOVE) INSTEAD
ExclusiveArch: noarch x86_64
ExclusiveOS: Linux
Provides: kernel-rt = %{rpmversion}
Provides: kernel-rt-drm = 4.3.0
Provides: kernel-rt-%{_target_cpu} = %{rpmversion}-%{pkg_release}

%kernel_reqprovconf
%ifarch x86_64
Obsoletes: kernel-smp
%endif

#
# List the packages used during the kernel build
#
BuildRequires: module-init-tools, patch >= 2.5.4, bash >= 2.03, sh-utils, tar
BuildRequires: bzip2, findutils, gzip, m4, perl, make >= 3.78, diffutils, gawk
BuildRequires: gcc >= 3.4.2, binutils >= 2.12, redhat-rpm-config
BuildRequires: net-tools, patchutils, rpm-build >= 4.8.0-7, bc
%if %{builddoc}
BuildRequires: xmlto
BuildRequires: asciidoc
%endif
%if %{with_sparse}
BuildRequires: sparse >= 0.4.1
%endif
%if %{buildrt}
%if %{buildperf}
BuildRequires: elfutils-libelf-devel zlib-devel binutils-devel newt-devel python-devel perl(ExtUtils::Embed)
BuildRequires: elfutils-devel asciidoc xmlto
%endif
%endif
%if %{signmodules}
BuildRequires: gnupg
%endif
BuildRequires: python

BuildConflicts: rhbuildsys(DiskFree) < 7Gb

%if %{fancy_debuginfo}
# Fancy new debuginfo generation introduced in Fedora 8.
# Note this must be a define since it needs to be overridden in some macro
# expansions
%define debuginfo_args --strict-build-id
%endif

Source0: %{name}-%{rpmversion}-%{pkg_release_simple}.tar.bz2

Source11: genkey
Source12: perf
Source13: perf-archive
Source14: find-provides
Source15: merge.pl
Source17: sanity_check.py
Source19: extrakeys.pub

Source20: %{name}-%{rpmversion}-x86_64-rt.config
Source21: %{name}-%{rpmversion}-x86_64-trace.config
Source22: %{name}-%{rpmversion}-x86_64-debug.config
Source23: %{name}-%{rpmversion}-x86_64-vanilla.config
Source24: %{name}-%{rpmversion}-x86_64-virt.config

### Started using a unified SRPM


# empty final patch file to facilitate testing of kernel patches
Patch999999: linux-kernel-test.patch

BuildRoot: %{_tmppath}/kernel-%{KVERREL}-root

# Override find_provides to use a script that provides "kernel(symbol) = hash".
# Pass path of the RPM temp dir containing kabideps to find-provides script.
# %global _use_internal_dependency_generator 0
# %global __find_provides %_sourcedir/find-provides %{_tmppath}
# %global __find_requires /usr/lib/rpm/redhat/find-requires kernel

%description
The kernel-rt package contains the Linux kernel (vmlinuz), the core of any
Linux operating system.  The kernel handles the basic functions
of the operating system: memory allocation, process allocation, device
input and output, etc.

This kernel has been compiled withe the RT patch applied and is intended
for use in deterministic response-time situations


%package doc
Summary: Various documentation bits found in the kernel source
Group: Documentation
%description doc
This package contains documentation files from the kernel
source. Various bits of information about the Linux kernel and the
device drivers shipped with it are documented in these files.

You'll want to install this package if you need a reference to the
options that can be passed to Linux kernel modules at load time.


%package headers
Summary: Header files for the Linux kernel for use by glibc
Group: Development/System
Obsoletes: glibc-kernheaders
Provides: glibc-kernheaders = 3.0-46
%description headers
Kernel-headers includes the C header files that specify the interface
between the Linux kernel and userspace libraries and programs.  The
header files define structures and constants that are needed for
building most standard programs and are also needed for rebuilding the
glibc package.

%package firmware
Summary: Firmware files used by the Linux kernel
Group: Development/System
# This is... complicated.
# Look at the WHENCE file.
License: GPL+ and GPLv2+ and MIT and Redistributable, no modification permitted
Provides: kernel-firmware = %{rpmversion}-%{pkg_release}
Provides: kernel-rt-firmware = %{rpmversion}-%{pkg_release}
%description firmware
Kernel-firmware includes firmware files required for some devices to
operate.

%if %{buildrelease}
%package -n mrg-rt-release
Summary:  MRG Realtime release string
Group:	  Documentation
License:  GPL+
Obsoletes: kernel-rt-release
Obsoletes: mrg-rt-release < %{rpmversion}-%{pkg_release}
Provides: mrg-rt-release = %{rpmversion}-%{pkg_release}
Provides: kernel-rt-release = %{rpmversion}-%{pkg_release}
%description -n mrg-rt-release
The MRG release string, which indicates the major, minor and errata release
of the realtime kernel
%endif

%if %{builddebuginfo}
%package debuginfo-common-%{_target_cpu}
Summary: Kernel source files used by %{name}-debuginfo packages
Group: Development/Debug
%description debuginfo-common-%{_target_cpu}
This package is required by %{name}-debuginfo subpackages.
It provides the kernel source files common to all builds.
%endif

%if %{buildrt}
%if %{buildperf}
%package -n perf-rt
Summary: Performance monitoring for the Linux kernel
Group: Development/System
Provides: perl(Perf::Trace::Context), perl(Perf::Trace::Core), perl(Perf::Trace::Util)
License: GPLv2
Obsoletes: perf
%description -n perf-rt
This package provides the supporting documentation for the perf tool
shipped in each kernel image subpackage.
%endif
%endif

#
# This macro creates a kernel-<subpackage>-debuginfo package.
#	%%kernel_debuginfo_package <subpackage>
#
%define kernel_debuginfo_package() \
%package %{?1:%{1}-}debuginfo\
Summary: Debug information for package %{name}%{?1:-%{1}}\
Group: Development/Debug\
Requires: %{name}-debuginfo-common-%{_target_cpu} = %{version}-%{release}\
Provides: %{name}%{?1:-%{1}}-debuginfo-%{_target_cpu} = %{version}-%{release}\
AutoReqProv: no\
%description -n %{name}%{?1:-%{1}}-debuginfo\
This package provides debug information for package %{name}%{?1:-%{1}}.\
This is required to use SystemTap with %{name}%{?1:%{1}}-%{KVERREL}.\
%{expand:%%global debuginfo_args %{?debuginfo_args} -p '/.*/%%{KVERREL}%{?1:\.%{1}}/.*|/.*%%{KVERREL}%{?1:\.%{1}}(\.debug)?' -o debuginfo%{?1}.list}\
%{nil}

#
# This macro creates a kernel-<subpackage>-devel package.
#	%%kernel_devel_package <subpackage> <pretty-name>
#
%define kernel_devel_package() \
%package %{?1:%{1}-}devel\
Summary: Development package for building kernel modules to match the %{?2:%{2} }kernel\
Group: System Environment/Kernel\
Provides: kernel-rt%{?1:-%{1}}-devel-%{_target_cpu} = %{version}-%{release}\
Provides: kernel-rt-devel-%{_target_cpu} = %{version}-%{release}%{?1:.%{1}}\
Provides: kernel-rt-devel = %{version}-%{release}%{?1:.%{1}}\
Provides: kernel-rt-devel-uname-r = %{KVERREL}%{?1:.%{1}}\
AutoReqProv: no\
Requires(pre): /usr/bin/find\
%description -n kernel-rt%{?variant}%{?1:-%{1}}-devel\
This package provides kernel headers and makefiles sufficient to build modules\
against the %{?2:%{2} }kernel package.\
%{nil}

#
# This macro creates a kernel-<subpackage> and its -devel and -debuginfo too.
#	%%define variant_summary The Linux kernel compiled for <configuration>
#	%%kernel_variant_package [-n <pretty-name>] <subpackage>
#
%define kernel_variant_package(n:) \
%package %1\
Summary: %{variant_summary}\
Group: System Environment/Kernel\
%kernel_reqprovconf\
%{expand:%%kernel_devel_package %1 %{!?-n:%1}%{?-n:%{-n*}}}\
%{expand:%%kernel_debuginfo_package %1}\
%{nil}


# First the auxiliary packages of the main kernel package.
%kernel_devel_package
%kernel_debuginfo_package

# Now, each variant package.

%if %{with debug}
# kernel-rt-debug
%define variant_summary The Linux kernel compiled with extra debugging enabled
%kernel_variant_package debug
%description debug
The kernel package contains the Linux kernel (vmlinuz), the core of any
Linux operating system.  The kernel handles the basic functions
of the operating system:  memory allocation, process allocation, device
input and output, etc.

This variant of the kernel has numerous debugging options enabled.
It should only be installed when trying to gather additional information
on kernel bugs, as some of these options impact performance noticably.
%endif

%if %{with trace}
# kernel-rt-trace
%define variant_summary The Linux kernel compiled with extra tracing options enabled
%kernel_variant_package trace
%description trace
The kernel package contains the Linux kernel (vmlinuz), the core of any
Linux operating system.  The kernel handles the basic functions
of the operating system:  memory allocation, process allocation, device
input and output, etc.

This variant of the kernel has numerous tracing options enabled.
It should only be installed when trying to gather additional information
on kernel bugs, as some of these options impact performance noticably.
%endif

%if %{with vanilla}
# kernel-rt-vanilla
%define variant_summary The Linux kernel compiled without RT capabilities for debugging
%kernel_variant_package vanilla
%description vanilla
The kernel package contains the Linux kernel (vmlinuz), the core of any
Linux operating system.  The kernel handles the basic functions
of the operating system:  memory allocation, process allocation, device
input and output, etc.

This variant of the kernel has the realtime capabilities disabled, but
has a configuration close to the one used in the realtime kernel.
It should only be installed when trying to gather additional information
on kernel bugs.
%endif

%if %{with virt}
# kernel-rt-virt
%define variant_summary The Linux kernel compiled with RT and virtualization
%kernel_variant_package virt
%description virt
The kernel package contains the Linux kernel (vmlinuz), the core of any
Linux operating system.  The kernel handles the basic functions
of the operating system:  memory allocation, process allocation, device
input and output, etc.

This variant of the kernel has both RT and virtualization options turned on.
It is strictly for internal testing and should never be delivered to a
customer.
%endif

%prep
## ApplyPatch routine
patch_command='patch -p1 -F1 -s'
ApplyPatch()
{
local patch=$1
shift
if [ ! -f $RPM_SOURCE_DIR/$patch ]; then
echo "Can't find $RPM_SOURCE_DIR/$patch"
exit 1;
fi
case "$patch" in
*.bz2) bunzip2 < "$RPM_SOURCE_DIR/$patch" | $patch_command ${1+"$@"} ;;
*.gz) gunzip < "$RPM_SOURCE_DIR/$patch" | $patch_command ${1+"$@"} ;;
*) $patch_command ${1+"$@"} < "$RPM_SOURCE_DIR/$patch" ;;
esac
}

# we don't want a .config file when building firmware: it just confuses the build system
%define build_firmware \
   # mv .config .config.firmware_save \
   make INSTALL_FW_PATH=$RPM_BUILD_ROOT/lib/firmware/%{rpmversion}-%{pkg_release} firmware_install \
   # mv .config.firmware_save .config \
   ln -s /lib/firmware/%{rpmversion}-%{pkg_release} $RPM_BUILD_ROOT/lib/firmware/%{rpmversion}-%{pkg_release}.i686 \
   ln -s /lib/firmware/%{rpmversion}-%{pkg_release} $RPM_BUILD_ROOT/lib/firmware/%{rpmversion}-%{pkg_release}.x86_64 \
   for kvname in debug trace; do \
      ln -s /lib/firmware/%{rpmversion}-%{pkg_release} $RPM_BUILD_ROOT/lib/firmware/%{rpmversion}-%{pkg_release}.i686.$kvname \
      ln -s /lib/firmware/%{rpmversion}-%{pkg_release} $RPM_BUILD_ROOT/lib/firmware/%{rpmversion}-%{pkg_release}.x86_64.$kvname \
   done

if [ ! -d kernel-%{kversion}/vanilla-%{kversion}/ ]; then
	rm -f pax_global_header;
%setup -q -n kernel-%{kversion} -c
	mv %{name}-%{rpmversion}-%{pkg_release_simple} vanilla-%{kversion};
else
	cd kernel-%{kversion}/;
fi

if [ -d linux-%{kversion}.%{_target_cpu} ]; then
  # Just in case we ctrl-c'd a prep already
  rm -rf deleteme.%{_target_cpu}
  # Move away the stale away, and delete in background.
  mv linux-%{kversion}.%{_target_cpu} deleteme.%{_target_cpu}
  rm -rf deleteme.%{_target_cpu} &
fi

cp -rl vanilla-%{kversion} linux-%{kversion}.%{_target_cpu}

cd linux-%{kversion}.%{_target_cpu}

## Apply Patches here
ApplyPatch linux-kernel-test.patch

# move off upstream version mechanism
if [ -e localversion-rt ]; then
   mv -f localversion-rt upstream-localversion-rt
fi

# Drop some necessary files from the source dir into the buildroot
#cp $RPM_SOURCE_DIR/config-* .
#cp %{SOURCE15} %{SOURCE17} .

# Dynamically generate kernel .config files from config-* files
#make -f %{SOURCE20} VERSION=%{version} configs

# Any further pre-build tree manipulations happen here.

chmod +x scripts/checkpatch.pl

# only deal with configs if we are going to build for the arch
%ifnarch %nobuildarches

mkdir configs

cp %{SOURCE20} %{SOURCE21} %{SOURCE22} %{SOURCE23} %{SOURCE24} .

# hack to use a new config for the rt-vanilla kernel
#cp %{SOURCE21} kernel-%{version}-x86_64-rtvanilla.config

# Remove configs not for the buildarch
#for cfg in kernel-rt-%{version}-*.config; do
#  if [ `echo %{all_arch_configs} | grep -c $cfg` -eq 0 ]; then
#    rm -f $cfg
#  fi
#done

%if !%{builddebug}
rm -f kernel-rt-%{version}-*debug.config
%endif

# now run oldconfig over all the config files
for i in *.config
do
  echo running oldconfig on $i
  mv $i .config
  Arch=`head -1 .config | cut -b 3-`
  ls -ls .config
  make ARCH=$Arch %{oldconfig_target} >/dev/null
  echo "# $Arch" > configs/$i
  cat .config >> configs/$i
  rm -f include/generated/kernel.arch
  rm -f include/generated/kernel.cross
done
# end of kernel config
%endif

# get rid of unwanted files resulting from patch fuzz
find . \( -name "*.orig" -o -name "*~" \) -exec rm -f {} \; >/dev/null

%if %{signmodules}
cp %{SOURCE19} .
cat <<EOF
###
### Now generating a PGP key pair to be used for signing modules.
###
### If this takes a long time, you might wish to run rngd in the background to
### keep the supply of entropy topped up.  It needs to be run as root, and
### should use a hardware random number generator if one is available, eg:
###
###     rngd -r /dev/hwrandom
###
### If one isn't available, the pseudo-random number generator can be used:
###
###     rngd -r /dev/urandom
###
EOF
gpg --homedir . --batch --gen-key %{SOURCE11}
cat <<EOF
###
### Key pair generated.
###
EOF
# if there're external keys to be included
if [ -s %{SOURCE19} ]; then
	gpg --homedir . --no-default-keyring --keyring kernel.pub --import %{SOURCE19}
fi
gpg --homedir . --export --keyring ./kernel.pub Red > extract.pub
gcc -o scripts/bin2c scripts/bin2c.c
scripts/bin2c ksign_def_public_key __initdata <extract.pub >crypto/signature/key.h
%endif

cd ..

###
### build
###
%build

%if %{with_sparse}
%global sparse_mflags	C=1
%endif

%if %{fancy_debuginfo}
# This override tweaks the kernel makefiles so that we run debugedit on an
# object before embedding it.  When we later run find-debuginfo.sh, it will
# run debugedit again.  The edits it does change the build ID bits embedded
# in the stripped object, but repeating debugedit is a no-op.  We do it
# beforehand to get the proper final build ID bits into the embedded image.
# This affects the vDSO images in vmlinux, and the vmlinux image in bzImage.
export AFTER_LINK=\
'sh -xc "/usr/lib/rpm/debugedit -b $$RPM_BUILD_DIR -d /usr/src/debug \
				-i $@ > $@.id"'
%endif

cp_vmlinux()
{
  eu-strip --remove-comment -o "$2" "$1"
}

BuildKernel() {
    MakeTarget=$1
    KernelImage=$2
    Flavour=$3
    InstallName=${4:-vmlinuz}

    # Pick the right config file for the kernel we're building
    Config=kernel-rt-%{version}-%{_target_cpu}-${Flavour:-rt}.config
    DevelDir=/usr/src/kernels/%{KVERREL}${Flavour:+.${Flavour}}

    # When the bootable image is just the ELF kernel, strip it.
    # We already copy the unstripped file into the debuginfo package.
    if [ "$KernelImage" = vmlinux ]; then
      CopyKernel=cp_vmlinux
    else
      CopyKernel=cp
    fi

    KernelVer=%{version}-%{release}.%{_target_cpu}${Flavour:+.${Flavour}}
    echo BUILDING A KERNEL FOR ${Flavour} %{_target_cpu}...

    # make sure EXTRAVERSION says what we want it to say
    perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = %{?stable_minor}-%{release}.%{_target_cpu}${Flavour:+.${Flavour}}/" Makefile

    # if pre-rc1 devel kernel, must fix up SUBLEVEL for our versioning scheme
    %if !0%{?rcrev}
    %if 0%{?gitrev}
    perl -p -i -e 's/^SUBLEVEL.*/SUBLEVEL = %{upstream_sublevel}/' Makefile
    %endif
    %endif

    # and now to start the build process

    make -s mrproper

    cp configs/$Config .config
    echo USING CONFIG configs/$Config

    Arch=`head -1 .config | cut -b 3-`
    echo USING ARCH=$Arch

    make -s ARCH=$Arch %{oldconfig_target} > /dev/null
    make -s ARCH=$Arch V=1 %{?_smp_mflags} $MakeTarget %{?sparse_mflags}
    make -s ARCH=$Arch V=1 %{?_smp_mflags} modules %{?sparse_mflags} || exit 1

%if %{buildrt}
%if %{buildperf}
    if [ -z "$Flavour" ]; then
        pushd tools/perf
        chmod +x util/generate-cmdlist.sh util/PERF-VERSION-GEN
        ### make -s %{?_smp_mflags} V=1 DESTDIR=$RPM_BUILD_ROOT prefix=/usr install
        make -s V=1 %{?_smp_mflags} NO_DEMANGLE=1 perf
        mkdir -p $RPM_BUILD_ROOT/usr/libexec/
        install -m 755 perf $RPM_BUILD_ROOT/usr/libexec/perf.%{KVERREL}%{?Flavour:+.%{Flavour}}
        popd
    fi
%endif
%endif

    # Start installing the results
%if %{builddebuginfo}
    mkdir -p $RPM_BUILD_ROOT%{debuginfodir}/boot
    mkdir -p $RPM_BUILD_ROOT%{debuginfodir}/%{image_install_path}
    install -m 644 System.map $RPM_BUILD_ROOT/%{debuginfodir}/boot/System.map-$KernelVer
%endif
    mkdir -p $RPM_BUILD_ROOT/%{image_install_path}
    install -m 644 .config $RPM_BUILD_ROOT/boot/config-$KernelVer
    install -m 644 System.map $RPM_BUILD_ROOT/boot/System.map-$KernelVer
%if %{use_dracut}
    # We estimate the size of the initramfs because rpm needs to take this size
    # into consideration when performing disk space calculations. (See bz #530778)
    dd if=/dev/zero of=$RPM_BUILD_ROOT/boot/initramfs-$KernelVer.img bs=1M count=20
%else
    dd if=/dev/zero of=$RPM_BUILD_ROOT/boot/initrd-$KernelVer.img bs=1M count=5
%endif
    if [ -f arch/$Arch/boot/zImage.stub ]; then
      cp arch/$Arch/boot/zImage.stub $RPM_BUILD_ROOT/%{image_install_path}/zImage.stub-$KernelVer || :
    fi
    $CopyKernel $KernelImage \
		$RPM_BUILD_ROOT/%{image_install_path}/$InstallName-$KernelVer
    chmod 755 $RPM_BUILD_ROOT/%{image_install_path}/$InstallName-$KernelVer

    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer
    # Override $(mod-fw) because we don't want it to install any firmware
    # We'll do that ourselves with 'make firmware_install'
    make -s ARCH=$Arch INSTALL_MOD_PATH=$RPM_BUILD_ROOT modules_install KERNELRELEASE=$KernelVer mod-fw=

      # check if the modules are being signed
%if %{signmodules}
    if [ -z "$(readelf -n $(find fs/ -name \*.ko | head -n 1) | grep module.sig)" ]; then
        echo "ERROR: modules are NOT signed" >&2;
        exit 1;
    fi
%endif

%ifarch %{vdso_arches}
    make -s ARCH=$Arch INSTALL_MOD_PATH=$RPM_BUILD_ROOT vdso_install KERNELRELEASE=$KernelVer
    if [ ! -s ldconfig-kernel.conf ]; then
      echo > ldconfig-kernel.conf "\
# Placeholder file, no vDSO hwcap entries used in this kernel."
    fi
    %{__install} -D -m 444 ldconfig-kernel.conf \
        $RPM_BUILD_ROOT/etc/ld.so.conf.d/kernel-rt-$KernelVer.conf
%endif

    # And save the headers/makefiles etc for building modules against
    #
    # This all looks scary, but the end result is supposed to be:
    # * all arch relevant include/ files
    # * all Makefile/Kconfig files
    # * all script/ files

    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/source
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    (cd $RPM_BUILD_ROOT/lib/modules/$KernelVer ; ln -s build source)
    # dirs for additional modules per module-init-tools, kbuild/modules.txt
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/extra
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/updates
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/weak-updates
    # first copy everything
    cp --parents `find  -type f -name "Makefile*" -o -name "Kconfig*"` $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    # now nuke the MRG directory
    # TODO: figure out of we need this or not
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/MRG

    cp System.map $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    if [ -s Module.markers ]; then
      cp Module.markers $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    fi

    # then drop all but the needed Makefiles/Kconfig files
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/Documentation
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    cp .config $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    cp -a scripts $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    if [ -d arch/$Arch/scripts ]; then
      cp -a arch/$Arch/scripts $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/arch/%{_arch} || :
    fi
    if [ -f arch/$Arch/*lds ]; then
      cp -a arch/$Arch/*lds $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/arch/%{_arch}/ || :
    fi
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts/*.o
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts/*/*.o

    # hack to remove .build-id conflict with RHEL kernel debuginfo
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts/conmakehash
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts/pnmtologo

    if [ -d arch/%{asmarch}/include ]; then
          cp -a --parents arch/%{asmarch}/include $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/
    fi
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    cd include
    cp -a $(echo * | sed -e 's/^asm$//') $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    #cp -a acpi config crypto keys linux math-emu media mtd net pcmcia rdma rxrpc scsi sound trace video drm asm-generic generated xen $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    if [ -L asm ]; then
      asmdir=$(readlink asm)
      cp -a $asmdir $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/
      pushd $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
      ln -s $asmdir asm
      popd
    fi
    # Make sure the Makefile and vermagic.h have a matching timestamp so that
    # external modules can be built
    touch -r $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/Makefile $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/linux/vermagic.h
    touch -r $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/.config $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/linux/autoconf.h
    # Copy .config to include/config/auto.conf so "make prepare" is unnecessary.
    cp $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/.config $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/config/auto.conf
    cd ..

    #
    # save the vmlinux file for kernel debugging into the kernel-debuginfo rpm
    #
%if %{builddebuginfo}
    mkdir -p $RPM_BUILD_ROOT%{debuginfodir}/lib/modules/$KernelVer
    cp vmlinux $RPM_BUILD_ROOT%{debuginfodir}/lib/modules/$KernelVer
%endif

    find $RPM_BUILD_ROOT/lib/modules/$KernelVer -name "*.ko" -type f >modnames

    # mark modules executable so that strip-to-file can strip them
    xargs --no-run-if-empty chmod u+x < modnames

    # Generate a list of modules for block and networking.
    fgrep /drivers/ modnames | xargs --no-run-if-empty nm -upA |
    sed -n 's,^.*/\([^/]*\.ko\):  *U \(.*\)$,\1 \2,p' > drivers.undef

    collect_modules_list()
    {
      sed -r -n -e "s/^([^ ]+) \\.?($2)\$/\\1/p" drivers.undef |
      LC_ALL=C sort -u > $RPM_BUILD_ROOT/lib/modules/$KernelVer/modules.$1
    }

    collect_modules_list networking \
			 'register_netdev|ieee80211_register_hw|usbnet_probe'
    collect_modules_list block \
			 'ata_scsi_ioctl|scsi_add_host|scsi_add_host_with_dma|blk_init_queue|register_mtd_blktrans|scsi_esp_register|scsi_register_device_handler'
    collect_modules_list drm \
			 'drm_open|drm_init'
    collect_modules_list modesetting \
			 'drm_crtc_init'

    # detect missing or incorrect license tags
    rm -f modinfo
    while read i
    do
      echo -n "${i#$RPM_BUILD_ROOT/lib/modules/$KernelVer/} " >> modinfo
      /sbin/modinfo -l $i >> modinfo
    done < modnames

    egrep -v \
	  'GPL( v2)?$|Dual BSD/GPL$|Dual MPL/GPL$|GPL and additional rights$' \
	  modinfo && exit 1

    rm -f modinfo modnames

    # remove files that will be auto generated by depmod at rpm -i time
    for i in alias alias.bin ccwmap dep dep.bin ieee1394map inputmap isapnpmap ofmap pcimap seriomap symbols symbols.bin usbmap
    do
      rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/modules.$i
    done

    # Move the devel headers out of the root file system
    mkdir -p $RPM_BUILD_ROOT/usr/src/kernels
    mv $RPM_BUILD_ROOT/lib/modules/$KernelVer/build $RPM_BUILD_ROOT/$DevelDir
    ln -sf ../../..$DevelDir $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    # copy Module.symvers
    cp Module.symvers $RPM_BUILD_ROOT/$DevelDir
}

###
# DO it...
###

# prepare directories
rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT/boot

cd linux-%{kversion}.%{_target_cpu}

%if %{builddebug}
BuildKernel %make_target %kernel_image debug
%endif

%if %{buildtrace}
BuildKernel %make_target %kernel_image trace
%endif

%if %{buildvanilla}
BuildKernel %make_target %kernel_image vanilla
%endif

%if %{buildvirt}
BuildKernel %make_target %kernel_image virt
%endif

# maybe %if %{buildrt}
%if %{buildrt}
BuildKernel %make_target %kernel_image
%endif

%if %{builddoc}
# Make the HTML and man pages.
make -j1 htmldocs mandocs || %{doc_build_fail}

# sometimes non-world-readable files sneak into the kernel source tree
chmod -R a=rX Documentation
find Documentation -type d | xargs chmod u+w
%endif

%if %{buildrt}
%if %{buildperf}
pushd tools/perf
make %{?_smp_mflags} man || %{doc_build_fail}
popd
%endif
%endif

###
### Special hacks for debuginfo subpackages.
###

# This macro is used by %%install, so we must redefine it before that.
%global debug_package %{nil}

%if %{fancy_debuginfo}
%define __debug_install_post \
  /usr/lib/rpm/find-debuginfo.sh %{debuginfo_args} %{_builddir}/%{?buildsubdir}\
%{nil}
%endif

%if %{builddebuginfo}
%ifnarch noarch
%global __debug_package 1
%files -f debugfiles.list debuginfo-common-%{_target_cpu}
%defattr(-,root,root)
%endif
%endif

###
### install
###

%install
# for some reason, on RHEL-5 RPM_BUILD_ROOT doesn't get set
if [ -z "$RPM_BUILD_ROOT" ]; then
	export RPM_BUILD_ROOT="%{buildroot}";
fi

cd linux-%{kversion}.%{_target_cpu}

%if %{buildrelease}
if [ ! -d $RPM_BUILD_ROOT/etc ]
then
	mkdir -p $RPM_BUILD_ROOT/etc
fi
echo "Red Hat MRG Realtime release" %{mrg_release} > $RPM_BUILD_ROOT/etc/mrg-realtime-release
%endif

%if %{builddoc}
docdir=$RPM_BUILD_ROOT%{_datadir}/doc/kernel-rt-doc-%{rpmversion}
man9dir=$RPM_BUILD_ROOT%{_datadir}/man/man9rt

# copy the source over
mkdir -p $docdir
tar -f - --exclude=man --exclude='.*' --exclude=*.xml --exclude=*.tmpl -c Documentation | tar xf - -C $docdir

# Install man pages for the kernel API.
mkdir -p $man9dir
find Documentation/DocBook/man -name '*.9.gz' -print0 |
xargs -0 --no-run-if-empty %{__install} -m 444 -t $man9dir $m
ls $man9dir | grep -q '' || > $man9dir/BROKEN
%endif # builddoc

# perf docs
%if %{buildrt}
%if %{buildperf}
mandir=$RPM_BUILD_ROOT%{_datadir}/man
man1dir=$mandir/man1
pushd tools/perf/Documentation
make install-man mandir=$mandir
popd

pushd $man1dir
for d in *.1; do
 gzip $d;
done
popd

# perf shell wrapper
mkdir -p $RPM_BUILD_ROOT/usr/sbin/
cp $RPM_SOURCE_DIR/perf $RPM_BUILD_ROOT/usr/sbin/perf
chmod 0755 $RPM_BUILD_ROOT/usr/sbin/perf
cp $RPM_SOURCE_DIR/perf-archive $RPM_BUILD_ROOT/usr/sbin/perf-archive
chmod 0755 $RPM_BUILD_ROOT/usr/sbin/perf-archive
mkdir -p $RPM_BUILD_ROOT%{_datadir}/doc/perf
%endif # buildperf
%endif

%if %{buildheaders}
# Install kernel headers
make ARCH=%{hdrarch} INSTALL_HDR_PATH=$RPM_BUILD_ROOT/usr headers_install

# Do headers_check but don't die if it fails.
make ARCH=%{hdrarch} INSTALL_HDR_PATH=$RPM_BUILD_ROOT/usr headers_check \
     > hdrwarnings.txt || :
if grep -q exist hdrwarnings.txt; then
   sed s:^$RPM_BUILD_ROOT/usr/include/:: hdrwarnings.txt
   # Temporarily cause a build failure if header inconsistencies.
   # exit 1
fi

find $RPM_BUILD_ROOT/usr/include \
     \( -name .install -o -name .check -o \
	-name ..install.cmd -o -name ..check.cmd \) | xargs rm -f

# glibc provides scsi headers for itself, for now
rm -rf $RPM_BUILD_ROOT/usr/include/scsi
rm -f $RPM_BUILD_ROOT/usr/include/asm*/atomic.h
rm -f $RPM_BUILD_ROOT/usr/include/asm*/io.h
rm -f $RPM_BUILD_ROOT/usr/include/asm*/irq.h
%endif

%if %{buildfirmware}
%{build_firmware}
mkdir -p $RPM_BUILD_ROOT%{_datadir}/doc/kernel-rt-%{rpmversion}-%{pkg_release}/firmware
cp firmware/WHENCE $RPM_BUILD_ROOT%{_datadir}/doc/kernel-rt-%{rpmversion}-%{pkg_release}/firmware
%endif

###
### clean
###

%clean
rm -rf $RPM_BUILD_ROOT

###
### scripts
###

#
# This macro defines a %%post script for a kernel*-devel package.
#	%%kernel_devel_post [<subpackage>]
#
%define kernel_devel_post() \
%{expand:%%post %{?1:%{1}-}devel}\
if [ -f /etc/sysconfig/kernel ]\
then\
    . /etc/sysconfig/kernel || exit $?\
fi\
if [ "$HARDLINK" != "no" -a -x /usr/sbin/hardlink ]\
then\
    (cd /usr/src/kernels/%{KVERREL}%{?1:.%{1}} &&\
     /usr/bin/find . -type f | while read f; do\
       hardlink -c /usr/src/kernels/*.fc*.*/$f $f\
     done)\
fi\
%{nil}

# This macro defines a %%posttrans script for a kernel package.
#	%%kernel_variant_posttrans [<subpackage>]
# More text can follow to go at the end of this variant's %%post.
#
# grubby might be called during installation when a boot loader configuration
# file is not present, so just drop any error messages. See BZ#610813 for
# more details.
%define kernel_variant_posttrans() \
%{expand:%%posttrans %{?1}}\
%{expand:\
NEWKERNARGS=""\
(/sbin/grubby --info=`/sbin/grubby --default-kernel`) 2>/dev/null | grep -q crashkernel\
if [ $? -ne 0 ]\
then\
	NEWKERNARGS="--kernel-args=\"crashkernel=auto\""\
fi\
NKPARGS="--mkinitrd --depmod --update"\
%if %{use_dracut}\
NKPARGS="$NKPARGS --dracut"\
%endif\
/sbin/new-kernel-pkg --package kernel-rt $NKPARGS %{KVERREL}%{?1:.%{1}} $NEWKERNARGS || exit $?\
}\
/sbin/new-kernel-pkg --package kernel-rt --rpmposttrans %{KVERREL}%{?1:.%{1}} || exit $?\
%{nil}

#
# This macro defines a %%post script for a kernel package and its devel package.
#	%%kernel_variant_post [-v <subpackage>] [-r <replace>]
# More text can follow to go at the end of this variant's %%post.
#
%define kernel_variant_post(v:r:) \
%{expand:%%kernel_devel_post %{?-v*}}\
%{expand:%%kernel_variant_posttrans %{?-v*}}\
%{expand:%%post %{?-v*}}\
%{expand:\
SETDEFAULT=""\
PKGNAME="--package %{!?-v:kernel-rt}%{?-v:kernel-rt-debughelper}"\
if ! /bin/grep "^UPDATEDEFAULT=no" /etc/sysconfig/kernel > /dev/null\
then\
	SETDEFAULT=%{!?-v:--make-default}\
	/bin/sed -r -i -e 's/^DEFAULTKERNEL=%{-r*}$/DEFAULTKERNEL=kernel-rt/' /etc/sysconfig/kernel || exit $?\
fi\
/sbin/new-kernel-pkg $PKGNAME $SETDEFAULT --install %{KVERREL}%{?-v:.%{-v*}} || exit $?\
}\
[ -d /lib/firmware/updates ] || mkdir -p /lib/firmware/updates\
ln -sf /lib/firmware/mrg-rt-firmware /lib/firmware/updates/%{KVERREL}%{?1:.%{1}}\
%{nil}


# This macro defines a %%preun script for a kernel package.
#	%%kernel_variant_preun <subpackage>
#
%define kernel_variant_preun() \
%{expand:%%preun %{?1}}\
/sbin/new-kernel-pkg --rminitrd --rmmoddep --remove %{KVERREL}%{?1:.%{1}} || exit $?\
%{nil}

# kernel-rt %post script
%kernel_variant_preun
%kernel_variant_post

%if %{with debug}
# kernel-rt-debug %post script
%kernel_variant_preun debug
%kernel_variant_post -v debug
%endif

%if %{with trace}
# kernel-rt-trace %post script
%kernel_variant_preun trace
%kernel_variant_post -v trace
%endif

%if %{with vanilla}
# kernel-rt-vanilla %post script
%kernel_variant_preun vanilla
%kernel_variant_post -v vanilla
%endif

%if %{with virt}
# kernel-rt-virt %post script
%kernel_variant_preun virt
%kernel_variant_post -v virt
%endif

if [ -x /sbin/ldconfig ]
then
    /sbin/ldconfig -X || exit $?
fi

###
### file lists
###

%if %{buildheaders}
%files headers
%defattr(-,root,root)
/usr/include/*
%endif

%if %{buildfirmware}
%files firmware
%defattr(-,root,root)
/lib/firmware/*
%doc %{_datadir}/doc/kernel-rt-%{rpmversion}-%{pkg_release}/firmware/WHENCE
%endif

# only some architecture builds need kernel-doc
%if %{builddoc}
%files doc
%defattr(-,root,root)
%{_datadir}/doc/kernel-rt-doc-%{rpmversion}/Documentation/*
%dir %{_datadir}/doc/kernel-rt-doc-%{rpmversion}/Documentation
%dir %{_datadir}/doc/kernel-rt-doc-%{rpmversion}
%{_datadir}/man/man9rt/*
%endif

%if %{buildrelease}
%files -n mrg-rt-release
%defattr(-,root,root)
/etc/mrg-realtime-release
%endif

%if %{buildrt}
%if %{buildperf}
%files -n perf-rt
%defattr(-,root,root)
/usr/sbin/perf
/usr/sbin/perf-archive
/usr/libexec/*
%{_datadir}/man/man1/*
%{_datadir}/doc/perf
%endif
%endif

# This is %{image_install_path} on an arch where that includes ELF files,
# or empty otherwise.
%global elf_image_install_path %{?kernel_image_elf:%{image_install_path}}

#
# This macro defines the %%files sections for a kernel package
# and its devel and debuginfo packages.
#	%%kernel_variant_files [-k vmlinux] <condition> <subpackage>
#
%define kernel_variant_files(k:) \
%if %{1}\
%{expand:%%files %{?2}}\
%defattr(-,root,root)\
/%{image_install_path}/%{?-k:%{-k*}}%{!?-k:vmlinuz}-%{KVERREL}%{?2:.%{2}}\
/boot/System.map-%{KVERREL}%{?2:.%{2}}\
/boot/config-%{KVERREL}%{?2:.%{2}}\
%dir /lib/modules/%{KVERREL}%{?2:.%{2}}\
/lib/modules/%{KVERREL}%{?2:.%{2}}/kernel\
/lib/modules/%{KVERREL}%{?2:.%{2}}/extra\
/lib/modules/%{KVERREL}%{?2:.%{2}}/build\
/lib/modules/%{KVERREL}%{?2:.%{2}}/source\
/lib/modules/%{KVERREL}%{?2:.%{2}}/updates\
/lib/modules/%{KVERREL}%{?2:.%{2}}/weak-updates\
%ifarch %{vdso_arches}\
/lib/modules/%{KVERREL}%{?2:.%{2}}/vdso\
/etc/ld.so.conf.d/kernel-rt-%{KVERREL}%{?2:.%{2}}.conf\
%endif\
/lib/modules/%{KVERREL}%{?2:.%{2}}/modules.*\
%if %{use_dracut}\
%ghost /boot/initramfs-%{KVERREL}%{?2:.%{2}}.img\
%else\
%ghost /boot/initrd-%{KVERREL}%{?2:.%{2}}.img\
%endif\
%{expand:%%files %{?2:%{2}-}devel}\
%defattr(-,root,root)\
%dir /usr/src/kernels\
/usr/src/kernels/%{KVERREL}%{?2:.%{2}}\
%if %{builddebuginfo}\
%ifnarch noarch\
%if %{builddebuginfo}\
%if %{fancy_debuginfo}\
%{expand:%%files -f debuginfo%{?2}.list %{?2:%{2}-}debuginfo}\
%{debuginfodir}/boot/System.map-%{KVERREL}%{?2:.%{2}}\
%else\
%{expand:%%files %{?2:%{2}-}debuginfo}\
%endif\
%endif\
%defattr(-,root,root)\
%if %{builddebuginfo}\
%if !%{fancy_debuginfo}\
%if "%{elf_image_install_path}" != ""\
%{debuginfodir}/%{elf_image_install_path}/*-%{KVERREL}%{?2:.%{2}}.debug\
%endif\
%endif\
%{debuginfodir}/lib/modules/%{KVERREL}%{?2:.%{2}}\
%{debuginfodir}/usr/src/kernels/%{KVERREL}%{?2:.%{2}}\
%{debuginfodir}/boot/System.map-%{KVERREL}%{?2:.%{2}}\
%endif\
%endif\
%endif\
%endif\
%{nil}

%kernel_variant_files %{buildrt}

%if %{builddebug}
%kernel_variant_files %{builddebug} debug
%endif

%if %{buildtrace}
%kernel_variant_files %{buildtrace} trace
%endif

%if %{buildvanilla}
%kernel_variant_files %{buildvanilla} vanilla
%endif

%if %{with virt}
%kernel_variant_files %{buildvirt} virt
%endif

%changelog
* Mon May 12 2014 John Kacur <jkacur@redhat.com> - 3.10.33-rt32.34
- Modified the .gitattributes file so that we don't pull in unwanted files when creating the tar.ball for the rpm
- floppy: ignore kernel-only members in FDRAWCMD ioctl input (1094314) {CVE-2014-1738} {CVE-2014-1737}
- floppy: don't write kernel-only members to FDRAWCMD ioctl output (1094314) {CVE-2014-1738} {CVE-2014-1737}
- ath9k: protect tid->sched check (1083255) {CVE-2014-2672}
- rds: prevent dereference of a NULL device in rds_iw_laddr_check (1083279) {CVE-2014-2678}
- mac80211: fix AP powersave TX vs. wakeup race (1083536) {CVE-2014-2706}
- net: fix for a race condition in the inet frag code (1070620) {CVE-2014-0100}
- net: ipv4: current group_info should be put after using (1087418) {CVE-2014-2851}
- mm: try_to_unmap_cluster() should lock_page() before mlocking (1093082) {CVE-2014-3122}
- n_tty: Fix n_tty_write crash when echoing in raw mode (1094238) {CVE-2014-0196}

* Wed Mar 26 2014 John Kacur <jkacur@redhat.com> - 3.10.33-rt32.33
- spec: do not create an empty include/linux/version.h
- spec: we no longer carry the bfa-3.1.2.1 firmware in the kernel
- spec: added ApplyPatch and the support for linux-kernel-test.patch
- spec: fix the buildid usage on the specfile
- sit: link local routes are missing (999371)

* Tue Mar 18 2014 John Kacur <jkacur@redhat.com> - 3.10.33-rt32.32
- Respin to add missing CVE information to the changelog

* Tue Mar 18 2014 John Kacur <jkacur@redhat.com> - 3.10.33-rt32.31
- netfilter: nf_conntrack_dccp: fix skb_header_pointer API usages (1077349) {CVE-2014-2523}

* Mon Mar 17 2014 John Kacur <jkacur@redhat.com> - 3.10.33-rt32.30
- add a dummy scsi_wait_scan module V2 (1076248)
- ipv6: don't set DST_NOCOUNT for remotely added routes (1075063) {CVE-2014-2309}

* Fri Mar 14 2014 John Kacur <jkacur@redhat.com> - 3.10.33-rt32.29
- Rebased to v3.10.33-rt32

* Thu Mar 06 2014 John Kacur <jkacur@redhat.com> - 3.10.32-rt31.28
- Rebased to v3.10.32-rt31
- cpu_chill: Add a UNINTERRUPTIBLE hrtimer_nanosleep (1072301)

* Tue Feb 26 2014 John Kacur <jkacur@redhat.com> - 3.10.27-rt25.27
- net: change TCP ack parameters for quicker timeouts on RT

* Tue Feb 25 2014 John Kacur <jkacur@redhat.com> - 3.10.27-rt25.26
- Revert crypto: aesni_intel - add more optimized XTS mode for x86-64 (1029560)
- Revert crypto: aesni_intel - fix Kconfig with CRYPTO_GLUE_HELPER_X86 (1029560)
- Revert crypto: aesni_intel - fix accessing of unaligned memory (1029560)

* Fri Feb 21 2014 John Kacur <jkacur@redhat.com> - 3.10.27-rt25.25
- hpsa: Update driver to 3.4.0-1 (1067880)

* Tue Feb 18 2014 John Kacur <jkacur@redhat.com> - 3.10.27-rt25.24
- cifs: sanity check data length before sending (1065670) {CVE-2014-0069}
- cifs: ensure uncached writes handle unmapped areas correctly (1065670) {CVE-2014-0069}

* Tue Feb 11 2014 John Kacur <jkacur@redhat.com> - 3.10.27-rt25.23
- turn off building the mrg-rt-release subpackage (handled in rt-setup)
- SELinux:  Fix kernel BUG on empty security contexts (1062506) {CVE-2014-1874}
- IB/cache: don't fill the cache with junk (1016735)
- IB/cma: Check for GID on listening device first (1016735)
- IB/cma: Use cached gids (1016735)

* Wed Feb 05 2014 John Kacur <jkacur@redhat.com> - 3.10.27-rt25.22
- rt: Make cpu_chill() use hrtimer instead of msleep (1061893)
- ip_output: do skb ufo init for peeked non ufo skb as well (1023494) {CVE-2013-4470}
- ip6_output: do skb ufo init for peeked non ufo skb as well (1023494) {CVE-2013-4470}
-
- We get the following upstream fixes by rebasing to more recent v3.10.x
- remove max_addresses check from ipv6_create_tempaddr (999371) {CVE-2013-0343}
- HID: logitech-dj: validate output report details (1000366) {CVE-2013-2895}
- kernel: HID: LG: heap overflow flaw (1000421) {CVE-2013-2893}
- HID: pantherlord: validate output report details (1000438) {CVE-2013-2892}
- HID: validate HID report id size (1000457) {CVE-2013-2888}
- HID: ntrig: validate feature report details (1000505) {CVE-2013-2896}
- net: flow_dissector: fail on evil iph->ihl (1007943) {CVE-2013-4348}
- dm snapshot: fix data corruption (1007950) {CVE-2013-4299}
- tuntap: correctly handle error in tun_set_iff() (1007952) {CVE-2013-4343}
- crypto: ansi_cprng - Fix off by one error in non-block size request (1009138) {CVE-2013-4345}
- net: sctp: fix ipv6 ipsec encryption bug in sctp_v6_xmit (1009673) {CVE-2013-4350}
- ipv6: udp packets following an UFO enqueued packet need also be handled by UFO (1011933) {CVE-2013-4387}
- ipc,sem: fine grained locking for semtimedop (1024871) {CVE-2013-4483}
- net: Update the sysctl permissions handler to test effective uid/gid (1027757) {CVE-2013-4270}
- perf/ftrace: Fix paranoid level for enabling function tracer (1027783) {CVE-2013-2930}
- aacraid: missing capable() check in compat ioctl (1033536) {CVE-2013-6383}
- libertas: potential oops in debugfs (1034182) {CVE-2013-6378}
- inet: prevent leakage of uninitialized memory to user in recv syscalls (1035886) {CVE-2013-7263}
- inet: fix addr_len/msg->msg_namelen assignment in recv_error and rxpmtu fun (1035886) {CVE-2013-7263}
- net: fib: fib6_add: fix potential NULL pointer dereference (1039071) {CVE-2013-6431 }
- exec/ptrace: fix get_dumpable() incorrect tests (1039489) {CVE-2013-2929}
- x86, fpu, amd: Clear exceptions in AMD FXSAVE workaround (1053598) {CVE-2014-1438}
- net: rework recvmsg handler msg_name and msg_namelen logic (1039872)
- netfilter: nf_nat: fix access to uninitialized buffer in IRC NAT helper (1058784)

* Thu Jan 30 2014 Luis Claudio R. Goncalves <lgoncalv@redhat.com> - 3.10.27-rt25.21
- revert: sit: fix bad list pointer (1058848)
- sit: add new fixes from Nicolas DichtelNicolas Dichtel (1058848)
- ip6_tunnel: add new fixes from Nicolas DichtelNicolas Dichtel (1058848)
- MRG: Update Makefile to clean 3.10 config files (not 3.8)

* Wed Jan 29 2014 Luis Claudio R. Goncalves <lgoncalv@redhat.com> - 3.10.27-rt25.20
- sit: revert commits 91762e7 9bc4c8c -- incomplete fixes (1058848)
- sit: fix bad list pointer (1058848)
- specfile: set kernel-rt as the default kernel on update (914765)

* Tue Jan 28 2014 John Kacur <jkacur@redhat.com> - 3.10.27-rt25.19
- sit: backport unregister part of add support of x-netns patch (1058848)

* Sun Jan 19 2014 Clark Williams <williams@redhat.com> - 3.10.27-rt25.18
- rebased to rt-stable 3.10.27-rt25

* Fri Jan 17 2014 Clark Williams <williams@redhat.com> - 3.10.26-rt24.17
- rebased to rt-stable 3.10.26-rt24

* Thu Jan 16 2014 Clark Williams <williams@redhat.com> - 3.10.25-rt23.16
- turn off LOCK_STAT in debug config until we can fix the problem
- specfile: respect UPDATEDEFAULT=no when installing kernel-rt
- config: enable CONFIG_NETFILTER_XT_MATCH_ADDRTYPE as a module
- jbd2: drop checkpoint mutex when waiting in __jbd2_log_wait_for_space()
- ip6tnl: fix use after free of fb_tnl_dev

* Tue Jan  7 2014 Clark Williams <williams@redhat.com> - 3.10.25-rt23.15
- rebased to rt-stable 3.10.25-rt23

* Fri Dec 20 2013 Clark Williams <williams@redhat.com> - 3.10.24-rt22.14
- bring back tcp_delack_min and tcp_ato_min tunables

* Wed Dec 18 2013 Clark Williams <williams@redhat.com> - 3.10.24-rt22.13
- sit: fix use after free of fb_tunnel_dev

* Wed Dec 18 2013 Clark Williams <williams@redhat.com> - 3.10.24-rt22.12
- actually rebase to 3.10.24 (rather than 3.10.22)

* Tue Dec 17 2013 Clark Williams <williams@redhat.com> - 3.10.24-rt22.11
- ip6tnl: fix use after free of fb_tnl_dev

* Tue Dec 17 2013 Clark Williams <williams@redhat.com> - 3.10.24-rt22.10
- rebased to linux-rt-stable 3.10.24-rt22
-
* Mon Dec 16 2013 Clark Williams <williams@redhat.com> - 3.10.22-rt19.9
- rebased to 3.10.22-rt19
- applied Gao Fengs patch to fix nflog panic

* Mon Nov 25 2013 Clark Williams <williams@redhat.com> - 3.10.20-rt17.8
- rebased to 3.10.20-rt17

* Wed Nov 13 2013 Clark Williams <williams@redhat.com> - 3.10.18-rt14.7
- reverted rostedt diagnostic patch

* Tue Nov 12 2013 Clark Williams <williams@redhat.com> - 3.10.18-rt14.6
- rebase to 3.10.18-rt14
- cherry-picked iwlwifi patch from 3.12-rt1
- turned on CONFIG_SPARSE_RCU_POINTER
- Applied rosted diagnostic patch for netfilter/rcu panics

* Wed Oct 30 2013 Clark Williams <williams@redhat.com> - 3.10.17-rt12.5
- rebase to 3.10.17-rt12

* Mon Oct 14 2013 Clark Williams <williams@redhat.com> - 3.10.15-rt11.4
- added MRG behavior changes (crashkernel=auto and NO_HZ off by default)

* Mon Oct 14 2013 Clark Williams <williams@redhat.com> - 3.10.15-rt11.3
- rebase to 3.10.15-rt11

* Tue Oct  8 2013 Clark Williams <williams@redhat.com> - 3.10.14-rt9.2
- rebase to 3.10.14-rt9

* Tue Sep  3 2013 Clark Williams <williams@redhat.com> - 3.10.10-rt7.1
- initial build
